/*
 * Copyright (C) 2018 CERN (www.cern.ch)
 * Author: Dimitris Lampridis <dimitris.lampridis@cern.ch>
 *
 * SPDX-License-Identifier: LGPL-3.0-or-later
 */

#include "mockturtle-rt.h"

#define ADCOUT_NUM_CHANNELS 5

struct wrtd_adcout_dev {
        uint32_t io_addr;
        /* FIXME: base channel (to create the event id).  */
};

static inline void adcout_writel(const struct wrtd_adcout_dev *dev,
                                 uint32_t value, uint32_t reg)
{
	dp_writel(value, dev->io_addr + reg);
}

static inline uint32_t adcout_readl(const struct wrtd_adcout_dev *dev, uint32_t reg)
{
	return dp_readl(dev->io_addr + reg);
}

static int adcout_init(struct wrtd_adcout_dev *adcout)
{
	pr_debug("%s: ADCout initialization complete\n\r", __func__);

	/* Disable all channels.  */
	adcout_writel(adcout, 0, ALT_TRIGOUT_CTRL);

	return 0;
}

static inline int adcout_wr_link_up(struct wrtd_adcout_dev *adcout)
{
	return adcout_readl(adcout, ALT_TRIGOUT_STATUS) & ALT_TRIGOUT_WR_LINK;
}

static inline int adcout_wr_time_locked(struct wrtd_adcout_dev *adcout)
{
	return 1;
}

static inline int adcout_wr_time_ready(struct wrtd_adcout_dev *adcout)
{
	return adcout_readl(adcout, ALT_TRIGOUT_STATUS) & ALT_TRIGOUT_WR_VALID;
}

static inline int adcout_wr_sync_timeout(void)
{
	return 0;
}

/**
 * Handles input timestamps from all TDC channels.
 *
 * TODO: tdc config: base address
 *                   first channel number (for sid)
 *                   time offset
 */
static void adcout_input(struct wrtd_adcout_dev *adcout)
{
        uint32_t status = adcout_readl(adcout, ALT_TRIGOUT_STATUS);
		uint32_t mask;
        struct wrtd_event ev;
		int i;

        /* Poll the FIFO and read the timestamp */
        if(!(status & ALT_TRIGOUT_TS_PRESENT))
            	return;

        /* FIXME: MSB word ?  */
        ev.ts.seconds = adcout_readl(adcout, ALT_TRIGOUT_TS_MASK_SEC + 0);
		mask = adcout_readl(adcout, ALT_TRIGOUT_TS_MASK_SEC + 4);
        ev.ts.ns = adcout_readl(adcout, ALT_TRIGOUT_TS_CYCLES) * 8;
        ev.ts.frac = 0;

		for (i = 0; i < ADCOUT_NUM_CHANNELS; i++) {
				if (!(mask & ((ALT_TRIGOUT_CH1_MASK >> 32) << i)))
						continue;
        		memset(ev.id, 0, WRTD_ID_LEN);
        		ev.id[0] = 'A';
        		ev.id[1] = 'D';
        		ev.id[2] = 'C';
				ev.id[3] = 'O';
        		ev.id[4] = '1' + i;
        		ev.flags = 0;

        		wrtd_log(WRTD_LOG_MSG_EV_GENERATED,
            		     WRTD_LOG_GENERATED_DEVICE + i, &ev, NULL);

        		/* Pass to wrtd.  */
        		wrtd_route_in(&ev);
		}
}
